#!/bin/bash

#------------------------------------------------------------------------------
# executes command (and terminates after timeout)
# -----------------------------------------------------------------------------

# arguments

TIMEOUT=${EPAGES_TIMEOUT:-5}
SIGNAL=TERM

usage() {
  echo "Usage:"
  echo "  $0 -c 'COMMAND' [ -t TIMEOUT ] [-s SIGNAL]"
  echo "Options:"
  echo "  -c: command to execute"
  echo "  -t: terminate command after TIMEOUT seconds (default: $EPAGES_TIMEOUT)"
  echo "  -s: terminate command with SIGNAL (not TRAP,ALRM; default: $SIGNAL)"
  echo "Example:"
  echo "  time $0 -c 'sleep 10' -t 1 -s KILL"
  exit 1
}

unset COMMAND PIDLIST proc
while getopts c:t:s: c ; do
  case $c in
    c) COMMAND=$OPTARG ;;
    t) TIMEOUT=$OPTARG ;;
    s) SIGNAL=$OPTARG ;;
    *) usage ;;
  esac
done

[ -z "$COMMAND" ] && usage

case "$SIGNAL" in
  HUP|INT|QUIT|ILL|IOT|BUS|FPE|KILL|USR1|SEGV|USR2|PIPE|ALRM|TERM|STKFLT|CHLD|CONT|STOP|TSTP|TTIN|TTOU|URG|XCPU|XFSZ|VTALRM|PROF|WINCH|IO|POLL|LOST|PWR|SYS|UNUSED) true ;;
  *) echo "$0: void SIGNAL" ; usage ;;
esac

# timeout_handler

timeout_handler () {
  timeout_code=127
  unset PIDLIST
  # get the process and all its children
  while [[ -n $proc ]] ; do
   PIDLIST="$proc$PIDLIST"
   proc=$(ps -opid= --ppid ${proc%,} | tr '\n' ',')
  done
  echo "$COMMAND: timeout after $TIMEOUT seconds (pid: $PIDLIST)"
  [[ -n $PIDLIST ]] || return
  kill -s $SIGNAL $PIDLIST >/dev/null 2>&1
}

# set trap on SIGALRM
unset timeout_code
trap timeout_handler ALRM

# start the alarm timer
# $pid is PID of this shell
pid=$$
( # sleep for $TIMEOUT seconds; if  we haven't been killed, send a SIGALRM to our caller
  sleep $TIMEOUT
  kill -s ALRM $pid
) &
# $alarm is the PID of the alarm subshell itself
alarm=$!

# run the command in background and get the PID in $proc so we can kill it if needed
eval "$COMMAND &"
proc=$!

# wait for the command to complete; interrupt if it times out
wait $proc
exit_code=$?

# the process exit status is captured in $pstatus, if it completed
# kill the alarm process, if it is still running (don't bother testing)
kill -s TERM $alarm >/dev/null 2>&1

# reset trap on SIGALRM
trap - ALRM

exit ${timeout_code:-$exit_code}
