#!/bin/bash

#==============================================================================
# httpd
#==============================================================================

fun_httpd_sourced=1

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

[[ -n $fun_epages_sourced ]] || . $(dirname $BASH_SOURCE)/fun-epages.sh
export etc_default component_installed

[[ -n $fun_host_sourced ]] || . $(dirname $BASH_SOURCE)/fun-host.sh
export get_host_name

[[ -n $fun_account_sourced ]] || . $(dirname $BASH_SOURCE)/fun-account.sh
export get_gid

[[ -n $fun_linux_sourced ]] || . $(dirname $BASH_SOURCE)/fun-linux.sh
export backup_file initial_file chkconfig_do

[[ -n $fun_arch_sourced ]] || . $(dirname $BASH_SOURCE)/fun-arch.sh
export EPAGES_KERNEL

#------------------------------------------------------------------------------
# provide
#------------------------------------------------------------------------------

export native_httpd_exe native_httpd_name httpd_name httpd_confd
export backup_httpd_confd HTTPD_ROOT EP_HTTPD EPAGES_REWRITE_CONF
export add_dummy_cert set_httpd_root httpd_ept2conf httpd_epx2conf
export get_httpd_user get_httpd_group get_httpd_groupid
export set_httpd_opts get_httpd_opts httpd_64 wi_ver disable_eproot_apache 
export get_httpd_sysconfig setup_etc_apache remove_etc_apache
export set_apache_envvars start_epages_apache mkdir_epages_apache
export set_httpd_modules httpd_modules_path get_apachectl
export set_selinux4httpd

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

#..............................................................................
# native_httpd_name
#..............................................................................
native_httpd_name() {
  local USAGE='
syntax: native_httpd_name
returns: name of native httpd daemon
'
  [[ $# -eq 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  if [[ -f /etc/debian_version ]] ; then
    [[ -f /etc/init.d/apache2 ]] && echo apache2
    return
  fi

  local HTTPD_RPM APACHE2_RPM

  rpm --quiet -q httpd && HTTPD_RPM=1
  rpm --quiet -q apache2 && APACHE2_RPM=1

  [[ -n $HTTPD_RPM && -f /etc/init.d/httpd ]] && { echo httpd; return; }
  [[ -n $APACHE2_RPM && -f /etc/init.d/apache2 ]] && { echo apache2; return; }

  [[ -f /etc/init.d/httpd ]] && { echo httpd; return; }
  [[ -f /etc/init.d/apache2 ]] && { echo apache2; return; }
}

#..............................................................................
# httpd_name
#..............................................................................
httpd_name() {
  local USAGE='
syntax: httpd_name
returns: name of httpd daemon
'
  [[ $# -eq 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  [[ -f /etc/init.d/apache2-epages ]] && { echo apache2-epages; return; }
  [[ -f /etc/init.d/httpd-epages ]] && { echo httpd-epages; return; }

  native_httpd_name
}

#..............................................................................
# set_selinux4httpd
#..............................................................................
set_selinux4httpd() {
  local USAGE='
syntax: set_selinux4httpd
description: sets SELinux boolean values for httpd
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  # return unless SELinux is installed
  chcon --version >/dev/null 2>&1 || return
  type -p setsebool 2>/dev/null || return

  # set SELinux boolean values for httpd
  for i in builtin_scripting can_network_connect can_network_relay enable_cgi use_nfs ; do
    setsebool httpd_$i on
  done
}

#..............................................................................
# get_httpd_sysconfig
#..............................................................................
get_httpd_sysconfig() {
  local USAGE='
syntax: get_httpd_sysconfig
description: echos the name of the file with httpd environment variables
  either /etc/$NATIVE_HTTPD/envvars or /etc/sysconfig/$NATIVE_HTTPD
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  local NATIVE_HTTPD=$(httpd_name)
  [[ -z $NATIVE_HTTPD ]] && return

  local SYSCONF=/etc/$NATIVE_HTTPD/envvars
  [[ -f $SYSCONF ]] || SYSCONF=$SYSCONF.sample
  [[ -f $SYSCONF ]] || SYSCONF=/etc/sysconfig/$NATIVE_HTTPD
  [[ -f $SYSCONF ]] && echo $SYSCONF
}

#..............................................................................
# get_httpd_opts
#..............................................................................
get_httpd_opts() {
  local USAGE='
syntax: get_httpd_opts [MORE_OPTIONS]
description: echos ENVVAR=ENVVAL
  - ENVVAR : name of variable containing web server arguments
  - ENVVAL : web server arguments (inclusive MORE_OPTIONS)
'
  [[ $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local MORE_OPTIONS=$@
  local NATIVE_HTTPD=$(httpd_name)
  [[ -z $NATIVE_HTTPD ]] && return

  local KEY VALUE
  local SYSCONF=$(get_httpd_sysconfig)
  [[ -z $SYSCONF ]] && return

  if [[ $SYSCONF =~ /etc/$NATIVE_HTTPD/envvars ]] ; then
    VALUE=$(/var/epages/ini-config.sh -f $SYSCONF -k APACHE_ARGUMENTS -G)
    KEY=APACHE_ARGUMENTS
  elif [[ $NATIVE_HTTPD = apache2 ]] ; then
    VALUE=$(. $SYSCONF
            [[ -z ${!APACHE_SERVER_FLAGS*} ]] && exit 1
            echo $APACHE_SERVER_FLAGS ) || return
    KEY=APACHE_SERVER_FLAGS
    MORE_OPTIONS=${MORE_OPTIONS//-D/}
  elif [[ -f /etc/sysconfig/$NATIVE_HTTPD ]] ; then
    VALUE=$(. $SYSCONF
            [[ -z ${!OPTIONS*} ]] && exit 1
            echo $OPTIONS ) || return
    KEY=OPTIONS
  fi

  [[ -z $KEY ]] && return

  VALUE=$(echo $VALUE $MORE_OPTIONS | tr ' ' '\n' | sort | uniq | tr '\n' ' ')
  echo "$KEY='$VALUE'"
}

#..............................................................................
# set_httpd_opts
#..............................................................................
set_httpd_opts() {
  local USAGE='
syntax: set_httpd_opts
description: adds HTTPD_OPTS from /etc/default/epages6 to web server arguments
'
  [[ $# -eq 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  local ETC_DEFAULT=$(readlink -f $(etc_default))
  [[ -f $ETC_DEFAULT ]] || { ls -l $ETC_DEFAULT; exit 1; } 
  . $ETC_DEFAULT

  local GET_HTTPD_OPTS=$(get_httpd_opts $HTTPD_OPTS)
  [[ -z ${GET_HTTPD_OPTS#*=} ]] && return

  eval $GET_HTTPD_OPTS
}

#..............................................................................
# set_httpd_modules
#..............................................................................
set_httpd_modules() {
  local USAGE='
syntax: set_httpd_modules MODULE_NAME...
description: sets envvar APACHE_MODULES=MODULE_NAME... in $(get_httpd_sysconfig)
'
  [[ $# = 0 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local FOUND_MODULES=$@
  local NATIVE_HTTPD=$(httpd_name)
  [[ -z $NATIVE_HTTPD ]] && return
  [[ $NATIVE_HTTPD = apache2 ]] || return

  local SYSCONF=$(get_httpd_sysconfig)
  [[ -z $SYSCONF ]] && return
  [[ $SYSCONF =~ /etc/$NATIVE_HTTPD/envvars ]] && return

  ( . $SYSCONF
    [[ -z ${!APACHE_MODULES*} ]] && exit 0
    am=" $APACHE_MODULES "
    for i in $FOUND_MODULES ; do
      [[ "${am/ $i /}" = "$am" ]] && APACHE_MODULES="$APACHE_MODULES $i"
    done
    [[ $am = " $APACHE_MODULES " ]] && exit 0
    /var/epages/ini-config.sh -f $SYSCONF -k APACHE_MODULES -v "\"$APACHE_MODULES\"" -i
  )
}

#..............................................................................
# httpd_confd
#..............................................................................
httpd_confd() {
  local USAGE='
syntax: httpd_confd [native] [epages]
returns: path to httpd confdir (with (native) epages configuration)
'
  local EPCONF NATIVE
  while [[ $# -gt 0 ]] ; do
    case $1 in
      epages) EPCONF=1 ;;
      native) NATIVE=1 ;;
      *) echo "$USAGE" 1>&2; return 1 ;;
    esac
    shift
  done

  local NATIVE_HTTPD
  [[ -n $NATIVE ]] && NATIVE_HTTPD=$(native_httpd_name) || NATIVE_HTTPD=$(httpd_name)

  [[ -z $NATIVE_HTTPD ]] && return
  [[ -d /etc/$NATIVE_HTTPD ]] || return

  for i in conf.d vhosts.d extra ; do
    if [[ -f "/etc/$NATIVE_HTTPD/$i/zzz-epages-httpd.conf" ]] ; then
      echo "/etc/$NATIVE_HTTPD/$i"
    elif [[ -d "/etc/$NATIVE_HTTPD/$i" && -z $EPCONF ]] ; then
      echo "/etc/$NATIVE_HTTPD/$i"
    fi
  done
}

#..............................................................................
# backup_httpd_confd
#..............................................................................
backup_httpd_confd() {
  local USAGE='
syntax: backup_httpd_confd [native] [epages]
description: copies /etc/httpd to /etc/httpd.backup-DATE (or so, dep on args)
returns: name of backup directory (/etc/httpd.backup-DATE)
'
  [[ $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local CONFD=$(httpd_confd $@)
  [[ -z $CONFD ]] && return
  local CONF_DIR=${CONFD%/*}
  local NOW=$(date '+%Y.%m.%d-%H.%M.%S')

  cp -rp $CONF_DIR $CONF_DIR.backup-$NOW
  ls -d $CONF_DIR.backup-$NOW
}

#..............................................................................
# HTTPD_ROOT EP_HTTPD EPAGES_REWRITE_CONF
#..............................................................................
EP_HTTPD= ; EPAGES_REWRITE_CONF=
if [[ -x ${EPAGES_APACHE:-/srv/epages/eproot/Apache}/bin/httpd && "$(httpd_confd)" = "" ]]
then
  HTTPD_ROOT=${EPAGES_APACHE:-/srv/epages/eproot/Apache}
  EP_HTTPD="$HTTPD_ROOT/bin/apachectl"
  EPAGES_REWRITE_CONF="$HTTPD_ROOT/conf/extra/zzz-epages-rewrite.conf"
else
  NATIVE_HTTPD=$(httpd_name)
  CONF_D=$(httpd_confd)
  HTTPD_ROOT=/etc/${NATIVE_HTTPD:-httpd}
  EPAGES_REWRITE_CONF=${CONF_D:-$HTTPD_ROOT/conf.d}/zzz-epages-rewrite.conf
fi

#..............................................................................
# disable_eproot_apache
#..............................................................................
disable_eproot_apache () {
  local USAGE='
syntax: disable_eproot_apache
description: stops eproot/Apache and moves it to eproot/Apache.obsolete-since-DATE
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  # return if no eproot/Apache found
  local HTTPD_EXE=${EPAGES_APACHE:-/srv/epages/eproot/Apache}/bin/httpd
  [[ -x $HTTPD_EXE ]] || return

  if ps auxwww | grep -q $HTTPD_EXE ; then
    if pgrep httpd-epages | grep -q . ; then
      pkill -9 httpd-epages >/dev/null 2>&1
    elif pgrep httpd | grep -q . ; then
      pkill -9 httpd >/dev/null 2>&1
    fi
  fi

  local NOW=$(date '+%Y.%m.%d-%H.%M.%S')
  mv ${EPAGES_APACHE:-/srv/epages/eproot/Apache} \
  ${EPAGES_APACHE:-/srv/epages/eproot/Apache}.obsolete-since-$NOW
}

#..............................................................................
# get_apachectl
#..............................................................................
get_apachectl() {
  local USAGE='
syntax: get_apachectl
description: echoes the path to the apachectl script (if exist)
'
  [[ $# -eq 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  [[ -n $EP_HTTPD ]] && { echo $EP_HTTPD; return; }

  for i in 2ctl-epages 2ctl ctl ; do
    PATH=$PATH:/usr/sbin:/sbin:/usr/bin:/bin type -p apache$i && break
  done
}

#..............................................................................
# get_httpd_user
#..............................................................................
get_httpd_user() {
  local USAGE='
syntax: get_httpd_user
description: echoes the user which runs the guess httpd server
'
  [[ $# -eq 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  local NATIVE_HTTPD=$(httpd_name)
  local SYSCONF=$(get_httpd_sysconfig)
  local RUN_USER

  if [[ -n $NATIVE_HTTPD && $SYSCONF =~ /etc/$NATIVE_HTTPD/envvars ]] ; then
    RUN_USER=$(/var/epages/ini-config.sh -f $SYSCONF -k APACHE_RUN_USER -G)
    [[ $RUN_USER = ${RUN_USER//'$'/} ]] || RUN_USER=
  fi

  if [[ -z $RUN_USER && -n $NATIVE_HTTPD && -d /etc/$NATIVE_HTTPD ]] ; then
    RUN_USER=$(find /etc/$NATIVE_HTTPD -type f -name "*.conf" -exec grep -P '\s*User\s' {} \; | head -1 | awk '{print $2}')
  fi

  local TRY_USER user
  case $EPAGES_KERNEL in
    *r) TRY_USER=apache ;;
    *s) TRY_USER=wwwrun ;;
    *d) TRY_USER=www-data ;;
  esac

  for user in $RUN_USER $TRY_USER apache wwwrun www-data ; do
    id -un $user 2>/dev/null && return
  done
  echo ${RUN_USER:-$TRY_USER}
}

#..............................................................................
# get_httpd_group
#..............................................................................
get_httpd_group() {
  local USAGE='
syntax: get_httpd_group
description: echoes the group which runs the guess httpd server
'
  [[ $# -eq 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  local NATIVE_HTTPD=$(httpd_name)
  local SYSCONF=$(get_httpd_sysconfig)
  local RUN_GROUP

  if [[ -n $NATIVE_HTTPD && $SYSCONF =~ /etc/$NATIVE_HTTPD/envvars ]] ; then
    RUN_GROUP=$(/var/epages/ini-config.sh -f $SYSCONF -k APACHE_RUN_GROUP -G)
    [[ $RUN_GROUP = ${RUN_GROUP//'$'/} ]] || RUN_GROUP=
  fi

  if [[ -z $RUN_GROUP && -n $NATIVE_HTTPD && -d /etc/$NATIVE_HTTPD ]] ; then
    RUN_GROUP=$(find /etc/$NATIVE_HTTPD -type f -name "*.conf" -exec grep -P '\s*Group\s' {} \; | head -1 | awk '{print $2}')
  fi

  local user=$(get_httpd_user)
  local TRY_GROUP group
  [[ -n $user ]] && TRY_GROUP=$(id -gn $user 2>/dev/null)

  if [[ -z $TRY_GROUP ]] ; then
    case $EPAGES_KERNEL in
      *r) TRY_GROUP=apache ;;
      *s) TRY_GROUP=www ;;
      *d) TRY_GROUP=www-data ;;
    esac
  fi

  for group in $RUN_GROUP $TRY_GROUP apache www www-data ; do
    getent group $group | grep -q . && { echo $group; return; }
  done
  echo ${RUN_GROUP:-$TRY_GROUP}
}

#..............................................................................
# get_httpd_groupid
#..............................................................................
get_httpd_groupid() {
  local USAGE='
syntax: get_httpd_groupid
description: echoes the group ID which runs the guess httpd server
'
  [[ $# -eq 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  local GROUP_ID=$(get_gid $(get_httpd_group))
  [[ -n $GROUP_ID ]] && { echo $GROUP_ID; return; }

  case $EPAGES_KERNEL in
    *r) echo 48 ;;
    *s) echo 8 ;;
    *d) echo 33 ;;
  esac
}

#..............................................................................
# native_httpd_exe
#..............................................................................
native_httpd_exe() {
  local USAGE='
syntax: native_httpd_exe
description: echoes the path to httpd server executable
'
  [[ $# -eq 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  for i in httpd httpd2 apache2 ; do
    [[ -e /usr/sbin/$i ]] && { echo /usr/sbin/$i; return 0; }
  done

  local HTTPD_EXE=`ls /usr/sbin/httpd2* 2>/dev/null | head -1`
  if [[ -n "$HTTPD_EXE" ]] ; then
    ln -s $HTTPD_EXE /usr/sbin/httpd2
    echo /usr/sbin/httpd2; return 0
  fi

  for i in httpd httpd2 apache2 ; do
    type -p $i 2>/dev/null && return 0
  done
}

#..............................................................................
# httpd_64
#..............................................................................
httpd_64() {
  local USAGE='
syntax: httpd_64
description: echoes 64 if $(native_httpd_exe) is 64-bit
'
  [[ $# -eq 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  local HTTPD_EXE
  HTTPD_EXE=$(native_httpd_exe) || return 1
  file -L $HTTPD_EXE | grep -q 'ELF 64-bit' && echo 64
}

#..............................................................................
# wi_ver
#..............................................................................
wi_ver() {
  local USAGE='
syntax: wi_ver
description: echoes webinterface version to use:
  if kernel version is less than 3.10 then use 2.2.8, else use 2.4.6
'
  [[ $# -eq 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  local UNAME_R=$(uname -r | awk -F. '{printf "%d%03d", $1, $2}')
  # if 'uname -r' is less than 3.10 then use WI for apache 2.2.8
  # else use WI for apache 2.4.6
  [[ $UNAME_R -lt 3010 ]] && echo 2.2.8 || echo 2.4.6
}

#..............................................................................
# httpd_modules_path
#..............................................................................
httpd_modules_path() {
  local USAGE='
syntax: httpd_modules_path
description: echoes the path to httpd modules directory
'
  [[ $# -eq 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  case $EPAGES_KERNEL in
    *r) echo /usr/lib$(httpd_64)/httpd/modules ;;
    *s) echo /usr/lib$(httpd_64)/apache2 ;;
    *d) echo /usr/lib/apache2/modules ;;
  esac
}

#..............................................................................
# add_dummy_cert
#..............................................................................
add_dummy_cert() {
  local USAGE='
syntax: add_dummy_cert [WS_HOST]
description: adds a dummy certificate for web server WS_HOST
'
  [[ $# -gt 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  . $(etc_default)

  local WS_HOST=$1
  if [[ -z $1 ]] ; then
    WS_HOST=$(get_host_name WS) || return 0
    WS_HOST=$(resolve_host_info WS)
    WS_HOST=${WS_HOST%% *}
  else
    R=-r
  fi

  /usr/sbin/dummycert.sh $R -h $WS_HOST -c "$HTTPD_ROOT" >/dev/null 2>&1
}

#..............................................................................
# set_httpd_root
#..............................................................................
set_httpd_root () {
  local USAGE='
syntax: set_httpd_root
description: sets httpd variables for .ept/.epx files in $HTTPD_ROOT/bin
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  pushd "$HTTPD_ROOT"/bin >/dev/null || return

  for i in $(ls -d *.ept 2>/dev/null) ; do
    backup_file "$i"
    sed "s,=HTTPD_ROOT=,$HTTPD_ROOT," "$i" > "${i%.ept}"
  done
  [ -e httpd-epages ] && rm -f httpd-epages
  ln -s httpd httpd-epages

  popd >/dev/null

  find "$HTTPD_ROOT" -name '*.epx' ! -name '*.conf.epx' | while read i ; do
    backup_file "$i"
    sed "s,_EPAGES_,$EPAGES,g" "$i" > "${i%.epx}"
  done
}

#..............................................................................
# httpd_ept2conf
#..............................................................................
httpd_ept2conf () {
  local USAGE='
syntax: httpd_ept2conf FILE(.ept)
description: sets httpd variables in FILE as defd in /etc/sysconfig/epages6
'
  [[ $# != 1 ]] || { echo "$USAGE" 1>&2; return 1; }

  local CONF_FILE=${1%.ept}
  local EPT_FILE=$CONF_FILE.ept

  [[ ! -f "$EPT_FILE" && ! -f "$CONF_FILE" ]] && { ls "$1" ;  echo "$USAGE" 1>&2; return 1; }

  backup_file "$EPT_FILE"

  local RW_CONF=$EPAGES_REWRITE_CONF

  . $(etc_default)
  . /etc/sysconfig/epages6

  local LOG_DIR=/var/log/$(httpd_name)
  local SERVER_ALIAS="ServerAlias $(get_host_name WS)"

  local LIB64=/lib$(httpd_64)
  [[ $LIB64 = /lib ]] && LIB64=

  sed -i "
  s, logs/, $LOG_DIR/,g;
  s,_EPAGES_WEBSECURE_,${EPAGES_WEBSECURE:-$EPAGES/Shared/WebSecure},g;
  s,_EPAGESJ_HOST_,$QUOTED_EPAGESJ_HOST,g;
  s,_EPAGESJ_PORT_,$EPAGES_INSTALL_EPAGESJ_PORT,g;
  s,_CONF_DIR_,$HTTPD_ROOT,g;
  s,_EPAGES_PERL_,$EPAGES_PERL,g;
  s,_EPAGES_,$EPAGES,g;
  s,_SERVERALIAS_,$SERVER_ALIAS,g;
  s,_HOST_,$EPAGES_INSTALL_WS_ADDR,g;
  s,_GROUP_,$EPAGES_INSTALL_WS_GROUP,g;
  s,_USER_,$EPAGES_INSTALL_WS_RUNUSER,g;
  s,_PORT_,$EPAGES_INSTALL_WS_PORT,g;
  s,_SSLPORT_,$EPAGES_INSTALL_WS_SSL_PORT,g;
  s,_LOCALPORT_,$EPAGES_INSTALL_WS_LOCAL_PORT,g;
  s,_LOCALSSLPORT_,$EPAGES_INSTALL_WS_LOCAL_SSLPORT,g;
  s,_REWRITE_CONF_,$RW_CONF,g;
  s,_IP_,$EPAGES_INSTALL_WS_IP,g;
  s,_KERNEL_,$EPAGES_KERNEL,g;
  s,_LIB_,$LIB64,g;
  s,_MODULES_,modules,g;
  " "$CONF_FILE"
}

#..............................................................................
# httpd_epx2conf
#..............................................................................
httpd_epx2conf () {
  local USAGE='
syntax: httpd_epx2conf DIR
description: sets $EPAGES in epx files in DIR
'
  local EPX_DIR=$1

  [[ $# = 0 ]] && { echo "$USAGE" 1>&2; return 1; }
  [[ ! -d "$EPX_DIR" ]] && { ls "$1" ;  echo "$USAGE" 1>&2; return 1; }

  . $(etc_default)

  find "$EPX_DIR" -name '*.epx' | while read i ; do
    backup_file "$i"
    sed "s,_EPAGES_,$EPAGES,g" "$i" > "${i%.epx}"
  done
}

#..............................................................................
# setup_etc_apache
#..............................................................................
setup_etc_apache () {
  local USAGE='
syntax     : setup_etc_apache NAME
description: create new instance of apache2
example    : setup_etc_apache epages
'
  [[ $# -eq 0 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local SUFFIX=$1

  for i in mod_disk_cache mod_evasive pagespeed_cache pagespeed_files ; do
    [[ -d /var/cache/apache2-$SUFFIX/$i ]] && continue
    echo Creating: /var/cache/apache2-$SUFFIX/$i
    mkdir -m 755 -p /var/cache/apache2-$SUFFIX/$i
  done
  chown -R $(get_httpd_user):$(get_httpd_group) /var/cache/apache2-$SUFFIX

  local MOD=$(httpd_modules_path)
  if [[ ! -f $MOD/mod_rpaf.so && -f $MOD/mod_rpaf-2.0.so ]] ; then
    echo Creating: $MOD/mod_rpaf.so
    ln $MOD/mod_rpaf-2.0.so $MOD/mod_rpaf.so
  fi
}

#..............................................................................
# remove_etc_apache
#..............................................................................
remove_etc_apache () {
  local USAGE='
syntax     : remove_etc_apache NAME
description: create new instance of apache2
example    : remove_etc_apache epages
'
  [[ $# -eq 0 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local SUFFIX=$1

  if [[ -d /var/log/apache2-$SUFFIX ]] ; then
    echo Removing /var/log/apache2-$SUFFIX
    rm -rf /var/log/apache2-$SUFFIX
  fi

  for i in mod_disk_cache mod_evasive pagespeed_cache pagespeed_files ; do
    [[ -d /var/cache/apache2-$SUFFIX/$i ]] || continue
    echo Removing /var/cache/apache2-$SUFFIX/$i
    rm -rf /var/cache/apache2-$SUFFIX/$i
  done
}

#..............................................................................
# set_apache_envvars
#..............................................................................
set_apache_envvars () {
  local USAGE='
syntax: set_apache_envvars
description: configure /etc/apache2-epages/envvars:
  - APACHE_SERVER_NAME
  - APACHE_SHORTURL_EXCEPT
  - APACHE_BASE_ALLOW
  - APACHE_MODS_DIR
  - APACHE_HTTPD
  - APACHE_PORT
  - APACHE_SSL_PORT
  - APACHE_LOCAL_PORT
  - APACHE_LOCAL_SSLPORT
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  local INI_CONFIG=/var/epages/ini-config.sh
  if [[ ! -x $INI_CONFIG ]] ; then
    echo "$BASH_SOURCE: script missing: $INI_CONFIG" 1>&2
    return 1
  fi

  local ETC_APACHE2=/etc/apache2-epages/envvars
  local ETC_DEFAULT=$(etc_default ${RPM_INSTALL_PREFIX:-/srv/epages})
  local ETC_INSTALL=/etc/sysconfig/epages6

  [[ -f $ETC_APACHE2 ]] || cp -p $ETC_APACHE2.sample $ETC_APACHE2 || return
  [[ -f "$ETC_DEFAULT" ]] && . "$ETC_DEFAULT"
  [[ -f $ETC_INSTALL ]] && . $ETC_INSTALL

  local HOST_INFO="$(resolve_host_info WS) localhost 127.0.0.1"

  local LIB64=/lib$(httpd_64)
  [[ $LIB64 = /lib ]] && LIB64=

  $INI_CONFIG -f $ETC_APACHE2 -k APACHE_SERVER_NAME -v "${HOST_INFO%% *}" -i
  $INI_CONFIG -f $ETC_APACHE2 -k APACHE_SHORTURL_EXCEPT -v "\"$HOST_INFO\"" -i
  $INI_CONFIG -f $ETC_APACHE2 -k APACHE_CLOSED_EXCEPT -v "\"$HOST_INFO\"" -i
  $INI_CONFIG -f $ETC_APACHE2 -k APACHE_BASE_ALLOW -v "\"$HOST_INFO\"" -i
  $INI_CONFIG -f $ETC_APACHE2 -k APACHE_MODS_DIR -v "\"$(httpd_modules_path)\"" -i
  $INI_CONFIG -f $ETC_APACHE2 -k APACHE_LIB64 -v "$LIB64" -i
  $INI_CONFIG -f $ETC_APACHE2 -k APACHE_VER -v "$(wi_ver)" -i
  $INI_CONFIG -f $ETC_APACHE2 -k APACHE_HTTPD -v "\"$(native_httpd_exe)\"" -i
  $INI_CONFIG -f $ETC_APACHE2 -k APACHE_PERL5LIB -v "\"$(epages_perl5lib)\"" -i
  $INI_CONFIG -f $ETC_APACHE2 -k APACHE_LD_LIBRARY_PATH -v "\"$(epages_libs)\"" -i

  local val
  for var in PORT SSL_PORT LOCAL_PORT LOCAL_SSLPORT ; do
    val=$(eval echo '$'EPAGES_INSTALL_WS_$var)
    [[ -z $val ]] && continue
    $INI_CONFIG -f $ETC_APACHE2 -k APACHE_$var -v "$val" -i
  done

  return 0
}

#..............................................................................
# start_epages_apache
#..............................................................................
start_epages_apache () {
  local USAGE='
syntax: start_epages_apache [start|stop|status|restart|reload|configtest]
description: adds and starts apache2-epages service
    with ARG just runs /etc/init.d/apache2-epages ARG (if script exist)
'
  local RUN=$1
  case $RUN in
    start|stop|status|restart|reload|configtest|'') true ;;
    *) echo "$USAGE" 1>&2; return 1 ;;
  esac

  local NATIVE_HTTPD=$(native_httpd_name)
  local EPAGES_HTTPD=$(httpd_name)

  if [[ -f /etc/init.d/$NATIVE_HTTPD ]] ; then
    /etc/init.d/$NATIVE_HTTPD stop >/dev/null 2>&1
    chkconfig_do --del $NATIVE_HTTPD >/dev/null 2>&1
  fi

  [[ $EPAGES_HTTPD = apache2-epages ]] || RUN=
  if [[ -z $RUN ]] ; then
    /etc/init.d/$EPAGES_HTTPD stop >/dev/null 2>&1
    chkconfig_do --add $EPAGES_HTTPD >/dev/null 2>&1
  fi
  /etc/init.d/$EPAGES_HTTPD ${RUN:-start}
}

#..............................................................................
# mkdir_epages_apache
#..............................................................................

is_local () {
  find "$@" -maxdepth 0 ! -fstype nfs 2>/dev/null | grep -q .
  return $?
}

mkdir_epages_apache () {
  local USAGE='
syntax: mkdir_epages_apache [first]
description: creates auxiliar apache directores with selinux context
    with argument first the selinux context is set recursively in webroot
'
  local FIRST
  [[ $1 = first ]] && FIRST=1
  [[ $# = 1 ]] && shift
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  . $(etc_default)

  [[ -n $APACHE_RUN_USER && -n $APACHE_RUN_GROUP ]] || return 0

  local run_user=$APACHE_RUN_USER
  local run_group=$APACHE_RUN_GROUP
  local run_dir=${APACHE_RUN_DIR:-/var/run/apache2-epages}
  local lock_dir=${APACHE_LOCK_DIR:-/var/lock/apache2-epages}
  local log_dir=${APACHE_LOG_DIR:-/var/log/apache2-epages}

  install -m 755 -d $run_dir $lock_dir $log_dir

  chcon --version >/dev/null 2>&1 || return 0
  type -p getenforce >/dev/null || return 0
  [[ "$(getenforce | tr A-Z a-z)" = enforcing ]] || return 0

  chcon -h system_u:object_r:httpd_log_t:s0 $log_dir
  chcon -L system_u:object_r:httpd_log_t:s0 $log_dir

  chcon -h system_u:object_r:httpd_lock_t:s0 $lock_dir
  chcon -RL system_u:object_r:httpd_lock_t:s0 $lock_dir

  chcon -h system_u:object_r:httpd_var_run_t:s0 $run_dir
  chcon -RL system_u:object_r:httpd_var_run_t:s0 $run_dir

  for i in "$EPAGES_CONFIG"/WebInterface.conf "$EPAGES_CONFIG"/ServerConfig.xml ; do
    [[ -f "$i" ]] || continue
    is_local "$i" && chcon system_u:object_r:httpd_config_t:s0 "$i"
  done
  if is_local "$EPAGES"/WebAdapter/ ; then
    chcon system_u:object_r:httpd_modules_t:s0 "$EPAGES"/WebAdapter/{lib64/,}*.so
  fi

  touch "$EPAGES_LOG"/WebAdapter.log
  if chown ${EPAGES_APPUSER:-eprunapp}:$run_group "$EPAGES_LOG"/WebAdapter.log ; then
    if is_local "$EPAGES_LOG"/WebAdapter.log ; then
      chcon system_u:object_r:httpd_log_t:s0 "$EPAGES_LOG"/WebAdapter.log
    fi
  else
    rm -f "$EPAGES_LOG"/WebAdapter.log
  fi
  if [[ $(ls -ldZ "$EPAGES_WEBROOT" | awk -F: '{print $3}') != httpd_sys_content_t ]] ; then
    if is_local "$EPAGES_WEBROOT" ; then
      chcon -h system_u:object_r:httpd_sys_content_t:s0 "$EPAGES_WEBROOT"
    fi
    if [[ -n $FIRST ]] && is_local "$EPAGES_WEBROOT"/ ; then
      chcon -RL system_u:object_r:httpd_sys_content_t:s0 "$EPAGES_WEBROOT"
    elif is_local "$EPAGES_WEBROOT"/ ; then
      chcon -L system_u:object_r:httpd_sys_content_t:s0 "$EPAGES_WEBROOT"
    fi
  fi
}

#------------------------------------------------------------------------------
# end
# -----------------------------------------------------------------------------
