#!/bin/bash

#==============================================================================
# java
#==============================================================================

fun_java_sourced=1

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

[[ -n $fun_arch_sourced ]] || . $(dirname $BASH_SOURCE)/fun-arch.sh
export EPAGES_KERNEL

[[ -n $fun_linux_sourced ]] || . $(dirname $BASH_SOURCE)/fun-linux.sh
export chkconfig_do

#------------------------------------------------------------------------------
# provide
#------------------------------------------------------------------------------

export java_executable jre_home mongod_name start_mongodb
export test_version set_alternative set_java_alternative

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

#..............................................................................
# test_version
#..............................................................................
test_version() {
  local USAGE='
syntax: test_version VERSION1 CMP VERSION2
description: returns true if VERSION1 CMP VERSION2 holds
arguments: CMP is one of (as in test): -lt -le -gt -ge -eq = ==
example: test_version 6.17.9 -lt 6.17.9.1
'
  [[ $# -ne 3 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }
  local VERSION1=$1 
  local CMP=$2
  local VERSION2=$3 

  # if CMP contains = or e (for equal) and versions are equal, return success
  [[ $CMP != ${CMP//[=e]/} && $VERSION1 = $VERSION2 ]] && return 0
  # if CMP does not contain = or e (for equal) and versions are equal, return failure
  [[ $CMP = ${CMP//[=e]/} && $VERSION1 = $VERSION2 ]] && return 1

  local EXPECTED
  case $CMP in
    =|==|-eq) return 1; ;;
    -lt|-le) EXPECTED=$VERSION1 ;;
    -gt|-ge) EXPECTED=$VERSION2 ;;
    *) echo "$USAGE" 1>&2; return 1; ;;
  esac
  [[ $(echo -e "$VERSION1\n$VERSION2" | sort -V | head -1) = $EXPECTED ]]
  return $?
}

#..............................................................................
# java_executable
#..............................................................................
java_executable() {
  local USAGE='
syntax: java_executable [-f] [6|7|8]
arguments: -f: return file that link points to instead of link
returns: java executable path (to java 6 resp. 7 or 8; default: latest)
'
  local SHOW_FILE SHOW
  [[ $1 = -f ]] && { SHOW_FILE=-f; shift; } || SHOW_FILE=
  [[ $# -gt 1 ]] && { echo "$USAGE" 1>&2; return 1; }
  [[ -n $1 && -n ${1/[1-9]/} ]] && { echo "$USAGE" 1>&2; return 1; }

  local FIND_VERSION=$1
  local JAVA_EXE

  if [[ -z $FIND_VERSION ]] ; then
    java_executable $SHOW_FILE 8 || java_executable $SHOW_FILE 7 || {
      #---------------------------------------------------------------------
      # last try: java is in $JRE_HOME or somewhere else
      #---------------------------------------------------------------------
      [[ -n $JRE_HOME && -x $JRE_HOME/bin/java ]] && { echo $JRE_HOME; return 0; }
      JAVA_EXE=$(type -p java) || return 1
      readlink -f $JAVA_EXE ; return $?
    }
    return 0
  fi

  #---------------------------------------------------------------------
  # first try: find java in link
  #---------------------------------------------------------------------
  local OPENJDK_LINK=/etc/alternatives
  local ORACLEJDK_LINK=/usr/java/latest/jre/bin
  local ORACLEJRE_LINK=/usr/java/latest/bin

  # try first openjdk, then oracle jdk, then oracle jre
  for i in $OPENJDK_LINK $OPENJDK_LINK/jre/bin $ORACLEJDK_LINK $ORACLEJRE_LINK ; do
    JAVA_EXE=$(readlink -f $i/java)
    [[ -x $JAVA_EXE ]] || continue
    [[ -n $SHOW_FILE ]] && SHOW=$JAVA_EXE || SHOW=$i/java
    # find java version
    [[ $JAVA_EXE = ${JAVA_EXE#*java-$FIND_VERSION} ]] || { echo $SHOW; return 0; }
    [[ $JAVA_EXE = ${JAVA_EXE#*java-1.$FIND_VERSION} ]] || { echo $SHOW; return 0; }
    [[ $JAVA_EXE = ${JAVA_EXE#*jdk1.$FIND_VERSION} ]] || { echo $SHOW; return 0; }
    [[ $JAVA_EXE = ${JAVA_EXE#*jre1.$FIND_VERSION} ]] || { echo $SHOW; return 0; }
  done

  #---------------------------------------------------------------------
  # second try: find java in link
  #---------------------------------------------------------------------
  for LIB in /usr/lib64 /usr/lib ; do
    JAVA_EXE=$(ls -v $LIB/jvm/java-$FIND_VERSION*/jre/bin/java 2>/dev/null | tail -1)
    [[ -n $JAVA_EXE && -x $JAVA_EXE ]] && { echo $JAVA_EXE; return 0; }
    JAVA_EXE=$(ls -v $LIB/jvm/java-1.$FIND_VERSION*/jre/bin/java 2>/dev/null | tail -1)
    [[ -n $JAVA_EXE && -x $JAVA_EXE ]] && { echo $JAVA_EXE; return 0; }
  done
  return 1
}

#..............................................................................
# jre_home
#..............................................................................
jre_home() {
  local USAGE='
syntax: jre_home [6|7|8]
returns: path to JRE directory (to java 6 resp. 7 or 8; default: latest)
'

  [[ $# -gt 1 ]] && { echo "$USAGE" 1>&2; return 1; }
  [[ -n $1 && -n ${1/[1-9]/} ]] && { echo "$USAGE" 1>&2; return 1; }

  local JAVA_EXE=$(java_executable -f $1)
  local EPAGES_JAVA=$(dirname "${JAVA_EXE%/*}")

  # $EPAGES_JAVA points to jre/ directory:
  # if $JAVA_EXE is /sth/jre/bin/java then $EPAGES_JAVA is /sth/jre
  [[ -n $JAVA_EXE ]] && echo "$EPAGES_JAVA" || return 1
}

#..............................................................................
# set_alternative
#..............................................................................
set_alternative() {
  local USAGE='
syntax: set_alternative link name path
description: executes: alternatives --install $link $name $path 1060
operating system: does not work on debian/ubuntu
'
  local link=$1
  local name=$2
  local path=$3

  [[ $# = 3 ]] || { echo "$USAGE" 1>&2; return 1; }

  [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%d} ]] || return 1

  # return if /etc/alternatives is not a directory
  pushd /etc/alternatives >/dev/null || return 1

  [[ -h $name ]] && alternatives --remove $name $(readlink -f $name)
  rm -f /var/lib/alternatives/$name
  alternatives --install $link $name $path 1060
  alternatives --set $name $path

  popd >/dev/null
}

#..............................................................................
# set_java_alternative
#..............................................................................
set_java_alternative() {
  local USAGE='
syntax: set_java_alternative [-test] [VERSION]
description: if installed java (java -version) is older than package
    openjdk-headless then updates /etc/alternatives with files from package
operating system: does not work on debian/ubuntu
arguments:
    VERSION: use java-VERSION-openjdk-headless package for setting /etc/alternatives
             (default: 1.8.0)
    -test: does nothing; returns success if /etc/alternatives is upgradable
example 1: updates /etc/alternatives only if outdated:
    set_java_alternative -test && set_java_alternative 
example 2: if both 1.7.0 and 1.8.0 are installed, switch between then:
    set_java_alternative 1.7.0
    set_java_alternative 1.8.0
'
  local DO_TEST
  [[ $1 = -test ]] && { DO_TEST=1; shift; }
  [[ $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  # return if operating system is debian
  [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%d} ]] || return 1

  local version=$1
  [[ -z $1 ]] && version=1.8.0
  local package=java-$version-openjdk

  # return if version of $package can't be found
  if ! rpm -q $package --list | grep -q '/jre/bin/java$' ; then
    package=$package-headless
    rpm -q $package --list | grep -q '/jre/bin/java$' || return 1
  fi
  local JDK_VERSION=$(rpm -q $package --qf '[%{version}]')

  # return if java executable can't be found
  type -p java >/dev/null || return 1

  local JAVA_VERSION=$(java -version 2>&1 | head -1 | sed 's,.* ,,;s,["'"'"'],,g')
  # return if version of java executable can't be found
  [[ $JAVA_VERSION =~ ^[0-9] ]] || return 1

  if [[ $DO_TEST ]] ; then
    # returns success if version of java executable < version of $package
    test_version ${JAVA_VERSION//[_-]/.} -lt $JDK_VERSION
    return $?
  fi

  rpm -q $package --list | while read file ; do
    depth=${file//[^\/]/}
    TARGET=
    # get java alternatives to set
    case $file in
      # java binaries:
      */jre/bin/*) TARGET=/usr/bin/${file##*/} ;;
      # java man pages:
      */man/man1/*) i=${file##*/} ; TARGET=/usr/share/man/man1/${i%%-*}.1.gz ;;
      # java directory /usr/lib/jvm/jre:
      */jre) [[ ${#depth} = 5 ]] || continue ; TARGET=/usr/lib/jvm/jre ;;
      # java directory /usr/lib/jvm-exports/jre:
      /usr/lib/jvm-exports/java-*) [[ ${#depth} = 4 ]] || continue ; TARGET=/usr/lib/jvm-exports/jre ;;
    esac

    # set java alternatives
    if [[ $TARGET =~ /usr/lib/jvm ]] ; then
      set_alternative $TARGET ${file##*/} $file
      set_alternative $TARGET-openjdk ${file##*/}_openjdk $file
    elif [[ $TARGET =~ / ]] ; then
      set_alternative $TARGET ${TARGET##*/} $file
    fi
    if  [[ $TARGET = /usr/lib/jvm-exports/jre ]] ; then
      # additional treatment of /usr/lib/jvm-exports/jre due to
      # java-$version-openjdk-headless installation error
      set_alternative $TARGET-$version-exports ${TARGET##*/}_${version}_exports $file
    fi
  done
  return 0
}

#..............................................................................
# mongod_name
#..............................................................................
mongod_name () {
  local USAGE='
syntax: mongod_name
description: return name of mongod as in init script
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }
  if [[ -f /etc/debian_version ]] ; then
    [[ -x /etc/init.d/mongodb ]] && echo mongodb
  elif grep -q epages /etc/init.d/mongod 2>/dev/null ; then
    echo mongod
  fi
}

#..............................................................................
# start_mongodb
#..............................................................................
start_mongodb () {
  local USAGE='
syntax: start_mongodb
description: adds and starts mongod service
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  if [[ ! -f /etc/debian_version ]] ; then
    [[ -x /etc/init.d/$(mongod_name) ]] && /etc/init.d/$(mongod_name) stop >/dev/null 2>&1
    pkill -9 -f /usr/bin/mongod

    cp -p /var/epages/mongod /etc/init.d

    if [ -f /etc/SuSE-release -a -f /etc/init.d/jexec ] ; then
      grep -q '^# Required-Stop:' /etc/init.d/jexec || {
        sed -i '/^# Required-Start:/a\
# Required-Stop:' /etc/init.d/jexec
      }
    fi

    chmod 750 /etc/init.d/mongod
    sed -i 's,chkconfig: -,chkconfig: 35,' /etc/init.d/mongod
    chkconfig_do --del mongod mongodb >/dev/null 2>&1
    chkconfig_do --add mongod >/dev/null 2>&1
    /etc/init.d/mongod stop >/dev/null 2>&1
  fi

  [[ -x /etc/init.d/$(mongod_name) ]] && /etc/init.d/$(mongod_name) start
}

#------------------------------------------------------------------------------
# end
# -----------------------------------------------------------------------------
