#!/bin/bash

#==============================================================================
# MySQL
#==============================================================================

fun_mysql_sourced=1

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

[[ -n $fun_epages_sourced ]] || . $(dirname $BASH_SOURCE)/fun-epages.sh
export etc_default

[[ -n $fun_host_sourced ]] || . $(dirname $BASH_SOURCE)/fun-host.sh
export my_host

#------------------------------------------------------------------------------
# provide
#------------------------------------------------------------------------------

export mysqld_name my_cnf mysql_installed mysqld_installed
export get_mysql_option get_mysql_connection set_mysql_connection

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

#..............................................................................
# mysqld_name
#..............................................................................
mysqld_name() {
  local USAGE='
syntax: mysqld_name
returns: name of MySQL daemon
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  local FPATH=$(systemctl --property=FragmentPath show mariadb 2>/dev/null)
  [[ -n ${FPATH#FragmentPath=} ]] && { echo mariadb; return; }

  [[ -f /etc/init.d/mysql ]] && { echo mysql; return; }
  [[ -f /etc/init.d/mysqld ]] && { echo mysqld; return; }
}

#..............................................................................
# my_cnf
#..............................................................................
my_cnf() {
  local USAGE='
syntax: my_cnf [$EPAGES_CONFIG/Database.d/SOME.conf]
returns: name of MySQL configuration file (from MySQLConfigFile=...) (if exist)
'
  [[ $# -gt 1 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  # mariadb has server config in /etc/my.cnf.d/server.cnf
  local MY_CNF=/etc/my.cnf.d/server.cnf
  # other mysqld have server config in /etc/(mysql/)?my.cnf
  [[ ! -f $MY_CNF && -f /etc/my.cnf ]] && MY_CNF=/etc/my.cnf
  [[ ! -f $MY_CNF && -f /etc/mysql/my.cnf ]] && MY_CNF=/etc/mysql/my.cnf
  
  [[ $# = 0 ]] && { [[ -f $MY_CNF ]] && echo $MY_CNF; return; }
  ls -d $1 >/dev/null || { echo "$USAGE" 1>&2; return 1; }

  local BU_MY_CNF=$(grep '^\s*MySQLConfigFile\s*=' $1 \
    | sed 's/.*MySQLConfigFile[[:space:]]*=[[:space:]]*//;s/["''"]//g;' | head -1)
  [[ -f ${BU_MY_CNF:-$MY_CNF} ]] && echo ${BU_MY_CNF:-$MY_CNF}
}

#..............................................................................
# mysql_installed
#..............................................................................
mysql_installed() {
  local USAGE='
syntax: mysql_installed
returns: 0 - MySQL client is installed ; 1 - not
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  cat $EPAGES_CONFIG/Database.d/*.conf 2>/dev/null \
  | grep -Pq '^\s*datasource\s*=\s*dbi:mysql' \
  && [[ -x ${EPSERVICE:-/srv/epages/eproot/Service/epagesService} ]]
  return $?
}

#..............................................................................
# mysqld_installed
#..............................................................................
mysqld_installed() {
  local USAGE='
syntax: mysqld_installed
returns: 0 - MySQL server is installed ; 1 - not
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  local file
  for file in $(ls $EPAGES_CONFIG/Database.d/*.conf 2>/dev/null) ; do
    datasource=$(grep -P '^\s*datasource\s*=\s*dbi:mysql' $file \
      | sed 's/.*datasource[[:space:]]*=[[:space:]]*//;s/["''"]//g;' | head -1)
    [[ -z $datasource ]] && continue
    [[ $datasource = ${datasource#*:*:*} ]] && continue
    host=${datasource##*hostname=}
    host=${host##*host=}
    if [[ $host != $datasource ]] ; then
      # is internet socket
      host=${host%%;*}
      my_host $host && return 0
    elif [[ $datasource != ${datasource#dbi:mysql:mysql_socket} ]] ; then
      # is file socket
      [[ -n $(my_cnf $file) ]] && return 0
    fi
  done
  return 1
}

. /etc/sysconfig/epages6

#..............................................................................
# get_mysql_option
#..............................................................................
get_mysql_option() {
  local USAGE='
syntax: [EPAGES_INSTALL_MY_CNF=...] get_mysql_option "SECTIONS" KEY DEFAULT
description: echoes value of first occurence of KEY in SECTIONS from file my.cnf
  (or EPAGES_INSTALL_MY_CNF if defined), else DEFAULT
examples:
  get_mysql_option "server mysqld" datadir /var/lib/mysql
  get_mysql_option "client mysql server mysqld" port 3306
'
  [[ $# -lt 2 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local MY_CNF_ARG result
  type -p my_print_defaults >/dev/null || { echo -n $3; return; }

  [[ -n $EPAGES_INSTALL_MY_CNF ]] && MY_CNF_ARG="-c $EPAGES_INSTALL_MY_CNF"
  for group in $1 ; do
    result=$(my_print_defaults $MY_CNF_ARG $group | sed -n "s/^--$2=//p" | tail -n 1)
    [[ -n $result ]] && { echo -n $result; return; }
  done
  echo -n $3
}

#..............................................................................
# get_mysql_connection
#..............................................................................
get_mysql_connection() {
  local USAGE='
syntax: MYSQL_CONN=($(get_mysql_connection))
description: echoes MySQL host and port derived from /etc/sysconfig/epages6 and my.cnf
example:
  MYSQL_CONN=($(get_mysql_connection)) ; HOST=${MYSQL_CONN[0]} ; PORT=${MYSQL_CONN[1]}
'
  [[ $# = 0 ]] || { echo "$USAGE" 1>&2; return 1; }

  local MYSQL_CONN=${EPAGES_MYSQL_CONNECTION:-$EPAGES_INSTALL_MYSQL_CONNECTION}
  # MYSQL_CONN can be either 'host;port' or 'host=host;port=port'
  MYSQL_CONN=$(echo $MYSQL_CONN | sed -r 's,[a-z]+=,,g')
  MYSQL_CONN=${MYSQL_CONN:-$EPAGES_INSTALL_MYSQL_HOST}

  local MYSQL_IP=$(my_ips | grep '\.' | head -1)
  local MYSQL_HOST=${EPAGES_INSTALL_MYSQL_HOST:-${MYSQL_CONN%;*}}
  MYSQL_HOST=$(get_mysql_option "client mysql" host ${MYSQL_HOST:-$MYSQL_IP})

  local MYSQL_PORT=${MYSQL_CONN#*;}
  [[ $MYSQL_CONN = $MYSQL_PORT ]] && MYSQL_PORT=
  MYSQL_PORT=$(get_mysql_option "client mysql server mysqld" port ${MYSQL_PORT:-3306})

  echo $MYSQL_HOST $MYSQL_PORT
}

#..............................................................................
# set_mysql_connection
#..............................................................................
set_mysql_connection() {
  local USAGE='
syntax: set_mysql_connection BU HOST PORT
description: sets dbi:mysql:host=HOST[;port=PORT] in \$EPAGES_CONFIG/Database.d/BU.conf
example:
  set_mysql_connection ASPoolDB $(get_mysql_connection)
'
  [[ $# -lt 2 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local DB_CONF="$EPAGES_CONFIG/Database.d/$1.conf"
  ls $DB_CONF >/dev/null || { echo "ERROR in: set_mysql_connection $@"; return 1; }

  local INI_CONFIG=/var/epages/ini-config.sh
  if [[ ! -x $INI_CONFIG ]] ; then
    echo "$BASH_SOURCE: script missing: $INI_CONFIG" 1>&2
    return 1
  fi

  local MYSQL_CONN="host=$2"
  [[ ${3:-3306} = 3306 ]] || MYSQL_CONN="$MYSQL_CONN;port=$3"

  $INI_CONFIG -f $DB_CONF -s Connection -k datasource -v "dbi:mysql:$MYSQL_CONN" -i
  chown ep_appl:epages $DB_CONF
}

#------------------------------------------------------------------------------
# end
# -----------------------------------------------------------------------------
