#!/bin/bash

#==============================================================================
# process functions
#==============================================================================

fun_process_sourced=1

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

[[ -n $fun_arch_sourced ]] || . $(dirname $BASH_SOURCE)/fun-arch.sh
export EPAGES_KERNEL

[[ -n $fun_term_sourced ]] || . $(dirname $BASH_SOURCE)/fun-term.sh
export COLUMNS

[[ -n $fun_linux_sourced ]] || . $(dirname $BASH_SOURCE)/fun-linux.sh
export chkconfig_do

#------------------------------------------------------------------------------
# provide
#------------------------------------------------------------------------------

export checkpid get_pidof get_pidfile get_pids show_pids show_daemon softkill

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

#..............................................................................
# checkpid
#..............................................................................
checkpid() {
  local USAGE='
syntax: checkpid PID...
description: returns 0 if at least one PID exists in /proc/, else 1
'
  [[ $# -eq 0 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local PID
  for PID in $* ; do
    [[ -d /proc/$PID ]] && return 0
  done
  return 1
}

#..............................................................................
# get_pidof
#..............................................................................
get_pidof() {
  local USAGE='
syntax: get_pidof PROGRAM
description: returns PIDs for PROGRAM by "pidof"
'
  [[ $# -eq 0 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local c=-c
  [[ $EPAGES_KERNEL = ${EPAGES_KERNEL%s} ]] || c=
  pidof $c -o $$ -o $PPID -o %PPID -x "$1" || pidof $c -o $$ -o $PPID -o %PPID -x "${1##*/}"
}

#..............................................................................
# get_pidfile
#..............................................................................
get_pidfile() {
  local USAGE='
syntax: get_pidfile [-p pidfile] [-u user] [/path/to/program]
description: get status of program (run by user) from /var/run pid file
returns:
  0 - program is running
  1 - program is dead and /var/run pid file exists
  3 - program is not running
  4 - user has insufficient privilege
  8 - unknown due to error
'
  [[ $# -eq 0 ]] && { echo "$USAGE" 1>&2; return 8; }
  local PID_FILE ACCOUNT
  while [ $# -gt 0 ] ; do
    case $1 in
      -p) PID_FILE=$2; shift 2 ;;
      -u) ACCOUNT=$2; shift 2 ;;
       *) break ;;
    esac
  done
  PID_FILE=${PID_FILE:-/var/run/${1##*/}.pid}

  local PID=
  if [[ -n $1 ]] ; then
    local ARGS=$(echo $1 | tr -d '"' | tr -d "'" | sed 's/  */ /g;s/[]\(|){}^$*+?[]/./g')
    [[ -n $ACCOUNT ]] && PID=$(pgrep -u "$ACCOUNT" -f "$ARGS") || PID=$(pgrep -f "$ARGS")
    if [[ -n $PID ]] ; then
      PID=$( (cat "$PID_FILE" 2>/dev/null ; echo "$PID") | sort | uniq )
      sh -c "echo \"$PID\" > \"$PID_FILE\"" 2>/dev/null
    fi
  fi

  [[ -s $PID_FILE ]] || return 3 # program is not running
  [[ -r $PID_FILE ]] || return 4 # user has insufficient privilege

  PID=
  while read LINE ; do
    [[ -z $LINE ]] && break
    [[ $LINE = $$ ]] && continue
    for i in $LINE ; do
      [[ -z ${p//[0-9]/} && -d /proc/$i ]] && PID="$PID $i"
    done
  done < "$PID_FILE"

  [[ -n ${PID# } ]] && { echo ${PID# }; return 0; }
  return 1 # program is dead and /var/run pid file exists
}

#..............................................................................
# get_pids
#..............................................................................
get_pids() {
  local USAGE='
syntax: get_pids {-c COMMAND | -p PID} [-u USER] [-C] [-P]
arguments:
  -c COMMAND: returns space separated PID list for command
  -p PID: returns space separated PID list for PID
  -u USER: returns only processes of USER
  -C: add child processes to PID list
  -P: add parent process to PID list
'
  [[ $# -eq 0 ]] && { echo "$USAGE" 1>&2; return 1; }

  local PARENTS CHILDREN COMMAND ACCOUNT PIDS PPIDS CPIDS PROCESS
  local OPTIND=0
  while getopts PCc:u:p:h? c $@ ; do
    case $c in
      P) PARENTS=1 ;;
      C) CHILDREN=1 ;;
      c) COMMAND=$OPTARG ;;
      u) ACCOUNT=$OPTARG ;;
      p) PROCESS=$OPTARG ;;
      [h?]) echo "$USAGE" 1>&2; return 1 ;;
    esac
  done
  [[ -z "$COMMAND$PROCESS" ]] && { echo "$USAGE" 1>&2; return 1; }

  if [[ -n $PROCESS ]] ; then
    PIDS=$(ps -p${PROCESS// /,} -opid= | sort | uniq | tr '\n' ',' | tr -d ' ')
  elif [[ -n $ACCOUNT ]] ; then
    PIDS=$(pgrep -u "$ACCOUNT" -f "$COMMAND" | tr '\n' ',')
  else
    PIDS=$(pgrep -f "$COMMAND" | tr '\n' ',')
  fi
  [[ -z ${PIDS%,} ]] && return

  if [[ -n $PARENTS ]] ; then
    PPIDS=$(ps -p${PIDS%,} -oppid= | sort | uniq | tr '\n' ',' | tr -d ' ')
  fi

  [[ -n $CHILDREN ]] && CPIDS=$(pgrep -P${PIDS%,} | tr '\n' ',')

  PIDS="$PPIDS$PIDS$CPIDS"
  echo ${PIDS//,/ }
}

#..............................................................................
# show_pids
#..............................................................................
show_pids() {
  local USAGE='
syntax: [PS_FORMAT=...] show_pids PID_LIST
arguments:
  PID_LIST: show processes of space separated PID_LIST in PS_FORMAT
'
  [[ $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }
  [[ $# -eq 0 ]] && return

  local PIDS=$@
  PS_FORMAT=${PS_FORMAT:-user,pid,ppid,nice,pri,pcpu,time,pmem,vsz,etime,args}
  [[ $# -gt 0 ]] && ps -p${PIDS// /,} -o$PS_FORMAT
}

#..............................................................................
# show_daemon
#..............................................................................
show_daemon() {
  local USAGE='
syntax: [PS_FORMAT=...] show_daemon DAEMON
arguments:
  DAEMON: show processes of daemon in PS_FORMAT
'
  [[ $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  local DAEMON=$1 ; shift
  [[ -z "$DAEMON" ]] && return
  chkconfig_do --list $DAEMON 2>/dev/null | grep -q . || return

  unset PIDS
  if LANG=C service $DAEMON status >/tmp/daemon.~$$~ ; then
    PIDS=$(awk '/pid/{gsub(/[^0-9]/,"",$3); print $3}' /tmp/daemon.~$$~)
  elif [[ $EPAGES_KERNEL != ${EPAGES_KERNEL%s} ]] ; then
    # suse
    for pidfile in $(grep -i '/var/.*\.pid' /etc/init.d/$DAEMON \
                     | sed -r 's,.*(/var/(\w|[+,./:=@~-])*).*,\1,' \
                     | grep -i '\.pid$') ; do
      [[ -s "$pidfile" ]] || continue
      PIDS=$(cat "$pidfile" | tr -cd '0-9')
      [[ -n $PIDS ]] && break
    done
  fi

  [[ -n ${PIDS//[^0-9]/} ]] && PIDS=$(get_pids -p "$PIDS" $@)
  if [[ -n ${PIDS//[^0-9]/} ]] ; then
    show_pids "$PIDS"
  elif [[ -f /tmp/daemon.~$$~ ]] ; then
    cat /tmp/daemon.~$$~
  fi
  rm -f /tmp/daemon.~$$~
}

#..............................................................................
# softkill
#..............................................................................
softkill() {
  local USAGE='
syntax: softkill PID_LIST
arguments:
  PID_LIST: space separated list of processes to kill; first try TERM, then KILL
'
  [[ $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }
  [[ $# -eq 0 ]] && return

  local PID_LIST=$@
  kill -15 $PID_LIST 2>/dev/null
  sleep 2

  PID_LIST=$(get_pids -p "$PID_LIST" -C)
  [[ -z $PID_LIST ]] && { echo SIGTERM; return; }

  kill -9 $PID_LIST 2>/dev/null
  sleep 2
  echo SIGKILL
}

#------------------------------------------------------------------------------
# end
# -----------------------------------------------------------------------------
