#!/bin/bash

#==============================================================================
# run script functions
#==============================================================================

fun_runscript_sourced=1

#------------------------------------------------------------------------------
# require
#------------------------------------------------------------------------------

[[ -n $fun_term_sourced ]] || . $(dirname $BASH_SOURCE)/fun-term.sh
export COLUMNS column em R G

[[ -n $fun_process_sourced ]] || . $(dirname $BASH_SOURCE)/fun-process.sh
export checkpid get_pidof get_pidfile get_pids

[[ -n $fun_arch_sourced ]] || . $(dirname $BASH_SOURCE)/fun-arch.sh
export u_sleep

#------------------------------------------------------------------------------
# provide
#------------------------------------------------------------------------------

export rs_show rs_uninvolved rs_status rs_start rs_stop

#------------------------------------------------------------------------------
# define
# -----------------------------------------------------------------------------

#..............................................................................
# rs_show
#..............................................................................
rs_show() {
  local RC=${1:-$?}
  local USAGE='
syntax: rs_show RETURN_CODE
description: show result of runscript
'
  [[ $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  if [[ $CONSOLE != /dev/console ]] ; then
    column 60
    [[ $RC = 0 ]] && RET="  $(G OK)  " || RET="$(R FAILED)"
    echo -e "[$RET]\r"
  fi
  return $RC
}

#..............................................................................
# rs_uninvolved
#..............................................................................
rs_uninvolved() {
  local USAGE='
syntax: rs_uninvolved DAEMON
description: show that runscript is uninvolved
'
  [[ $# = 0 || $1 != ${1#-} ]] && { echo "$USAGE" 1>&2; return 1; }

  if [[ $CONSOLE != /dev/console ]] ; then
    echo -n "Starting $1 daemon: "
    column 56 
    echo -e "[$(em -Y -d UNINVOLVED)]\r"
  fi
}

#..............................................................................
# rs_status
#..............................................................................
rs_status() {
  local USAGE='
syntax: rs_status [-p pidfile] [-l lockfile] [-u user] [program]
description: shows status of runscript (run by user)
returns:
  0 - program is running
  1 - program is dead and /var/run pid file exists
  3 - program is not running
  4 - user has insufficient privilege
  8 - unknown due to error
'
  [[ $# = 0 ]] && { echo "$USAGE" 1>&2; return 8; }

  local PID PID_FILE LOCK_FILE ACCOUNT
  # req. for ${!PID_FILE*} ${!LOCK_FILE*} later:
  unset PID_FILE LOCK_FILE
  while [ $# -gt 0 ] ; do
    case $1 in
      -p) PID_FILE=$2 ; shift 2 ;;
      -l) LOCK_FILE=$2 ; shift 2 ;;
      -u) ACCOUNT=$2; shift 2 ;;
       *) break ;;
    esac
  done

  if [[ ( -n ${!PID_FILE*} && -z $PID_FILE ) ||
        ( -n ${!LOCK_FILE*} && -z $LOCK_FILE ) ]] ; then
    echo "$USAGE" 1>&2; return 8
  fi

  if [[ -z $LOCK_FILE ]] ; then
    LOCK_FILE=${PID_FILE##*/}
    LOCK_FILE=/var/lock/subsys/${LOCK_FILE%.pid}
  fi

  local BASE=${1##*/}
  if [[ -n $PID_FILE ]] ; then
    BASE=${PID_FILE##*/} ; BASE=${BASE%.pid}
  fi

  # check /var/run/$PID_FILE.pid
  PID=$(get_pidfile -u "$ACCOUNT" -p "$PID_FILE" "$1")
  local RC=$?
  # check processes
  [ -z "$PID_FILE" -a -z "$PID" ] && PID=$(get_pidof $1)

  [[ -n $PID ]] && { echo "$BASE (pid $PID) is running..."; return 0; }

  case "$RC" in
    0) echo "$BASE (pid $PID) is running..." ; return 0 ;;
    1) echo "$BASE dead but pid file exists" ; return 1 ;;
    4) echo "$BASE status unknown due to insufficient privileges." ; return 4 ;;
  esac

  # see if /var/lock/subsys/$LOCK_FILE exists
  [[ -f $LOCK_FILE ]] && { echo "$BASE dead but subsys locked"; return 2; }

  echo "$BASE is stopped"
  return 3
}

#..............................................................................
# rs_start
#..............................................................................
rs_start() {
  local USAGE='
syntax: rs_start [-bg] [-q|-Q] [-l logfile] [-u user] [-p pidfile] [-i identifier] {program}
description: starts runscript (run by user)
'
  [[ $# = 0 ]] && { echo "$USAGE" 1>&2; return 1; }

  local PID PID_FILE RUN_USER REDIR1 REDIR2 LOG_FILE BG IDENTIFIER
  # req. for ${!PID_FILE*} ${!RUN_USER*} later: 
  unset PID_FILE RUN_USER
  while [ $# -gt 0 ] ; do
    case $1 in
      -i) IDENTIFIER=$2 ; shift 2 ;;
      -p) PID_FILE=$2 ; shift 2 ;;
      -u) RUN_USER=$2 ; shift 2 ;;
      -l) LOG_FILE=$2 ; shift 2 ;;
      -q) REDIR1=' >/dev/null' ; shift ;;
      -Q) REDIR2=' 2>/dev/null' ; shift ;;
      -bg) BG=' &' ; shift ;;
       *) break ;;
    esac
  done
  IDENTIFIER=${IDENTIFIER:-$1}

  if [[ ( -n ${!PID_FILE*} && -z $PID_FILE ) ||
        ( -n ${!RUN_USER*} && -z $RUN_USER ) ]] ; then
    echo "$USAGE" 1>&2; return 1
  fi

  if [[ $CONSOLE != /dev/console ]] ; then
    local DAEMON=${PID_FILE:-$IDENTIFIER}
    DAEMON=${DAEMON##*/}
    echo -n "Starting ${DAEMON%.pid} daemon: "
  fi

  if [[ -n $LOG_FILE ]] ; then
    mkdir -p $(dirname $LOG_FILE)
    if [[ -z "$REDIR1$REDIR2" || ( -n $REDIR1 && -n $REDIR2 ) ]] ; then
      REDIR1=" >>$LOG_FILE" ; REDIR2=' 2>&1'
    else
      [[ -n $REDIR1 ]] && REDIR1=" >>$LOG_FILE"
      [[ -n $REDIR2 ]] && REDIR2=" 2>>$LOG_FILE"
    fi
  fi

  # see if it's already running
  PID=$(get_pidfile -u "$RUN_USER" -p "$PID_FILE" "$IDENTIFIER")
  [ -n "$PID" ] && return

  # start it up
  if [[ -n $RUN_USER && $RUN_USER != root ]] ; then
    local SU_EXE=runuser
    $SU_EXE -c true 2>/dev/null || SU_EXE=su
    $SU_EXE $RUN_USER -c "$*$REDIR1$REDIR2$BG"
  else
    /bin/bash -c "$*$REDIR1$REDIR2$BG"
  fi
  rs_show $?

  # create pid file
  get_pidfile -u "$RUN_USER" -p "$PID_FILE" "$IDENTIFIER" >/dev/null
  return 0
}

#..............................................................................
# rs_stop
#..............................................................................
rs_stop() {
  local USAGE='
syntax: rs_stop [-p pidfile] [-d delay] [-u user] [-C command] [program [-signal]]
description: stops runscript (run by user)
'
  [[ $# = 0 ]] && { echo "$USAGE" 1>&2; return 1; }

  local PID PID_FILE DELAY COMMAND ACCOUNT
  # req. for ${!PID_FILE*} ${!DELAY*} later: 
  unset PID_FILE DELAY
  while [ $# -gt 0 ] ; do
    case $1 in
      -p) PID_FILE=$2 ; shift 2 ;;
      -d) DELAY=$2 ; shift 2 ;;
      -C) COMMAND=$2 ; shift 2 ;;
      -u) ACCOUNT=$2; shift 2 ;;
       *) break ;;
    esac
  done

  if [[ ( -n ${!PID_FILE*} && -z $PID_FILE ) ||
        ( -n ${!DELAY*} && -z $DELAY ) ]] ; then
    echo "$USAGE" 1>&2; return 1
  fi

  DELAY=${DELAY//[^0-9]/}
  [[ -n $DELAY ]] || DELAY=3
  local SIGNAL=$2

  if [[ $CONSOLE != /dev/console ]] ; then
    local DAEMON=${PID_FILE:-$1}
    DAEMON=${DAEMON##*/}
    echo -n "Shutting down ${DAEMON%.pid} daemon: "
  fi

  # save basename
  local BASE=${1##*/}
  # check /var/run/$PID_FILE.pid
  PID=$(get_pidfile -u "$ACCOUNT" -p "$PID_FILE" "$1")
  local RC=$?
  # check processes
  if [ -z "$PID" ]; then
    if [ -z "$PID_FILE" ]; then
      PID=$(get_pidof $1)
    else
      [ "$RC" = "4" ] && { rs_show $RC; return $RC ;}
    fi
  fi

  if [ -z "$PID" ]; then
    # program is not running
    [ -z "$SIGNAL" ] && {
      rs_show 1; RC=3; rm -f "${PID_FILE:-/var/run/$BASE.pid}";
    }
    return $RC
  fi

  [ -n "$COMMAND" ] && $COMMAND

  if [ -n "$SIGNAL" ] ; then
    # use specified level only
    if checkpid $PID; then
      kill $SIGNAL $PID >/dev/null 2>&1
      RC=$?
      rs_show $RC
    fi
    return $RC
  fi

  if checkpid $PID 2>&1 ; then
    # TERM first, then KILL if not dead
    kill -TERM $PID >/dev/null 2>&1
    u_sleep 100000
    WAITING=0
    while checkpid $PID ; do
      [[ $WAITING -ge $DELAY ]] && break
      sleep 1
      WAITING=$(($WAITING + 1))
    done
    kill -KILL $PID >/dev/null 2>&1
    u_sleep 100000
  fi
  checkpid $PID
  RC=$((! $?))
  rs_show $RC

  # remove pid file if any
  rm -f "${PID_FILE:-/var/run/$BASE.pid}"
  return $RC
}

#------------------------------------------------------------------------------
# end
# -----------------------------------------------------------------------------
