#!/bin/env bash

########################################################################
# usage
########################################################################

BN=${0##*/}

usage() {
  echo "Usage:"
  echo "  $BN { --set | --restore } [ --perl-cgi ] [ --no-start ] [ --verbose ]"
  echo "Options:"
  echo "  --set     : mount web server files with SELinux context"
  echo "  --restore : mount web server files with httpd_use_nfs on"
  echo "  --perl-cgi: set additionally SELinux context for Perl files"
  echo "  --no-start: do not start ePages at the end of the script"
  echo "  --verbose : verbose output"
  exit 1
}

PATH=$PATH:/sbin:/usr/sbin

QUIET=--quiet
unset SET RESTORE PERL_CGI VERBOSE NO_START
while [ $# -gt 0 ] ; do
  case $1 in
    --set) SET=1 ; shift ;;
    --restore) RESTORE=1 ; shift ;;
    --no-start) NO_START=1 ; shift ;;
    --perl-cgi) PERL_CGI=$1 ; shift ;;
    --verbose) VERBOSE=$1 ; unset QUIET ; shift ;;
    *) usage ;;
  esac
done

[[ -n $SET && -n $RESTORE ]] && usage
[[ -z $SET && -z $RESTORE ]] && usage

[[ -z $VERBOSE ]] && exec 2>/dev/null

########################################################################
# test arguments/environment
########################################################################

# test if chcon and getenforce are executable
[[ -n $VERBOSE ]] && echo -n "chcon --version: "
chcon --version >/dev/null || exit 0
[[ -n $VERBOSE ]] && echo -n "getenforce: "
type getenforce >/dev/null || exit 0
enforce=$(getenforce | tr A-Z a-z)
[[ -n $VERBOSE ]] && echo $enforce

# set epages environment
[[ ! -f /etc/sysconfig/epages6 ]] || . /etc/sysconfig/epages6
EPAGES_PROJECT=${EPAGES_PROJECT:-$EPAGES_INSTALL_RUN_SCRIPT}
EPAGES_PROJECT=${EPAGES_PROJECT:-epages6}

. /etc/default/$EPAGES_PROJECT
. /var/epages/function/fun-pkg.sh
export pkg_installed

# test if packages epages-webconf and policycoreutils are installed
pkg_installed webconf || exit 0

if [[ $EPAGES_KERNEL != ${EPAGES_KERNEL%d} ]] ; then
  # on debian
  dpkg --status policycoreutils  >/dev/null || exit 0
else
  # not on debian
  rpm $QUIET -q policycoreutils || exit 0
fi

# test if SELinux variable httpd_use_nfs exists
[[ -n $VERBOSE ]] && echo -n "getsebool httpd_use_nfs --> "
httpd_use_nfs=$(getsebool httpd_use_nfs | awk '{print $3}')
[[ -n $VERBOSE ]] && echo $httpd_use_nfs
if [[ -n $RESTORE && $enforce = enforcing && -z $httpd_use_nfs ]] ; then
  echo "$BN: Can't mount NFS w/o SELinux options because"
  echo "    SELinux is enforcing and no SELinux variable httpd_use_nfs exists"
  exit 1
fi

# test if mount supports SELinux
[[ -n $VERBOSE ]] && mount --version
mount --version | grep -qi selinux || {
  [[ -n $VERBOSE ]] && echo "mount: no selinux support"
  [[ -n $SET ]] && exit 0
}

# test if EPAGES_SHARED is NFS mounted
FILESERVER=$(awk -F: '/:.srv.epages.eproot.Shared/{print $1; exit}' /etc/fstab)
if [[ -z $FILESERVER ]] ; then
  echo "$BN: No regex :.srv.epages.eproot.Shared found in /etc/fstab; first run:"
  echo "    /var/epages/epages-fs.sh -mount FILESERVER"
  exit 1
fi

# test if FILESERVER has DNS entry
host $FILESERVER 1>&2 || exit 1

MOUNT_OPT='async,exec,noatime,nodev,nolock,bg,hard,intr,rw,rsize=16384,wsize=16384'

EPAGES_SHARED=/srv/epages/eproot/Shared
EPAGES_PERL=/srv/epages/eproot/Perl
EPAGES_CARTRIDGES=/srv/epages/eproot/Cartridges

########################################################################
# stop services
########################################################################

echo /etc/init.d/$EPAGES_PROJECT stop
/etc/init.d/$EPAGES_PROJECT stop >/dev/null

echo /etc/init.d/$EPAGES_PROJECT stop_java
/etc/init.d/$EPAGES_PROJECT stop_java >/dev/null

if [[ -n $SET ]] ; then

  ########################################################################
  # set EPAGES_SHARED
  ########################################################################

  # unmount EPAGES_SHARED
  if mount | grep -q " on $EPAGES_SHARED " ; then
    umount -f $EPAGES_SHARED 2>&1 || { fuser -vm $EPAGES_SHARED; exit 1; }
  fi
  ls -ld $EPAGES_SHARED/* 2>/dev/null && {
    echo "$BN: Directory must be empty: $EPAGES_SHARED/"
    exit 1
  }
  grep -v "^[^#]* $EPAGES_SHARED " /etc/fstab > /tmp/.~$$~ && cat /tmp/.~$$~ > /etc/fstab
  rm -f /tmp/.~$$~

  if [[ -e $EPAGES_SHARED ]] ; then
    rmdir $EPAGES_SHARED || {
      echo "$BN: Directory must be empty: $EPAGES_SHARED/"
      exit 1
    }
  fi
  if [[ -e $EPAGES_SHARED.nfs_t ]] ; then
    rmdir $EPAGES_SHARED.nfs_t || {
      echo "$BN: Directory must be empty: $EPAGES_SHARED.nfs_t/"
      exit 1
    }
  fi
  mkdir -p -m 755 $EPAGES_SHARED.nfs_t $EPAGES_SHARED || exit 1
  grep -q "^[^#]* $EPAGES_SHARED.nfs_t " /etc/fstab || {
    echo "$FILESERVER:$EPAGES_SHARED $EPAGES_SHARED.nfs_t nfs $MOUNT_OPT 0 0" >> /etc/fstab
  }

  # mount EPAGES_SHARED.nfs_t without SELinux context
  echo mount $EPAGES_SHARED.nfs_t
  mount $EPAGES_SHARED.nfs_t || exit 1

  # mount each directory in EPAGES_SHARED with its specific SELinux context
  declare -A FCONTEXT
  FCONTEXT[Config]=httpd_config_t
  FCONTEXT[Log]=httpd_log_t
  FCONTEXT[WebRoot]=httpd_sys_content_t
  FCONTEXT[Monitor]=bin_t

  cd $EPAGES_SHARED.nfs_t || exit 1
  for i in * ; do
    if [[ -n ${FCONTEXT[$i]} && ( $i != Monitor || -n $PERL_CGI )  ]] ; then
      grep -q "^[^#]* $EPAGES_SHARED/$i " /etc/fstab || {
        echo "$FILESERVER:$EPAGES_SHARED/$i $EPAGES_SHARED/$i nfs nosharecache,context=system_u:object_r:${FCONTEXT[$i]}:s0,$MOUNT_OPT 0 0" >> /etc/fstab
      }
      mount | grep -q " on $EPAGES_SHARED/$i " || {
        mkdir -p -m 755 $EPAGES_SHARED/$i || exit 1
        echo mount $EPAGES_SHARED/$i
        mount $EPAGES_SHARED/$i || exit 1
      }
    else
      export i
      echo ln -s $EPAGES_SHARED.nfs_t/$i
      ( cd $EPAGES_SHARED && ln -s $EPAGES_SHARED.nfs_t/$i )
      chown -h $(find $EPAGES_SHARED.nfs_t/$i -maxdepth 0 -printf "%u:%g") $i
    fi
  done

  ########################################################################
  # setsebool httpd_use_nfs off
  ########################################################################

  if [[ -n $httpd_use_nfs ]] ; then
    echo setsebool httpd_use_nfs off
    setsebool httpd_use_nfs off
  fi

  [[ -z $PERL_CGI ]] && exit 0

  ########################################################################
  # set EPAGES_CARTRIDGES
  ########################################################################

  # remount EPAGES_CARTRIDGES with its specific SELinux context
  if mount | grep -q " on $EPAGES_CARTRIDGES " ; then
    umount -f $EPAGES_CARTRIDGES 2>&1 || { fuser -vm $EPAGES_CARTRIDGES; exit 1; }
  fi
  grep -v "^[^#]* $EPAGES_CARTRIDGES " /etc/fstab > /tmp/.~$$~ && cat /tmp/.~$$~ > /etc/fstab
  rm -f /tmp/.~$$~

  echo "$FILESERVER:$EPAGES_CARTRIDGES $EPAGES_CARTRIDGES nfs nosharecache,context=system_u:object_r:usr_t:s0,$MOUNT_OPT 0 0" >> /etc/fstab

  echo mount $EPAGES_CARTRIDGES
  mount $EPAGES_CARTRIDGES || exit 1

  ########################################################################
  # set EPAGES_PERL
  ########################################################################

  # remount EPAGES_PERL with its specific SELinux context
  declare -A FCONTEXT=()
  FCONTEXT[bin]=bin_t
  FCONTEXT[lib]=textrel_shlib_t
  FCONTEXT[lib32]=lib_t

  if mount | grep -q " on $EPAGES_PERL " ; then
    umount -f $EPAGES_PERL 2>&1 || { fuser -vm $EPAGES_PERL; exit 1; }
  fi
  grep -v "^[^#]* $EPAGES_PERL " /etc/fstab > /tmp/.~$$~ && cat /tmp/.~$$~ > /etc/fstab
  rm -f /tmp/.~$$~

  for i in bin lib lib32 man ; do
    if [[ -n ${FCONTEXT[$i]} ]] ; then
      grep -q "^[^#]* $EPAGES_PERL/$i " /etc/fstab || {
        echo "$FILESERVER:$EPAGES_PERL/$i $EPAGES_PERL/$i nfs nosharecache,context=system_u:object_r:${FCONTEXT[$i]}:s0,$MOUNT_OPT 0 0" >> /etc/fstab
      }
    else
      grep -q "^[^#]* $EPAGES_PERL/$i " /etc/fstab || {
        echo "$FILESERVER:$EPAGES_PERL/$i $EPAGES_PERL/$i nfs $MOUNT_OPT 0 0" >> /etc/fstab
      }
    fi
    mount | grep -q " on $EPAGES_PERL/$i " || {
      mkdir -p -m 755 $EPAGES_PERL/$i || exit 1
      echo mount $EPAGES_PERL/$i
      mount $EPAGES_PERL/$i || exit 1
    }
  done

fi

if [[ -n $RESTORE ]] ; then

  ########################################################################
  # restore EPAGES_SHARED
  ########################################################################

  if mount | grep -q " on $EPAGES_SHARED.nfs_t " ; then
    umount -f $EPAGES_SHARED.nfs_t 2>&1 || { fuser -vm $EPAGES_SHARED.nfs_t; exit 1; }
  fi
  if [[ -d $EPAGES_SHARED.nfs_t ]] ; then
    rmdir $EPAGES_SHARED.nfs_t || {
      echo "$BN: Directory must be empty: $EPAGES_SHARED.nfs_t/"
      exit 1
    }
  fi
  grep -v "^[^#]* $EPAGES_SHARED.nfs_t " /etc/fstab > /tmp/.~$$~ && cat /tmp/.~$$~ > /etc/fstab
  rm -f /tmp/.~$$~

  find $EPAGES_SHARED/* -maxdepth 0 -type l -exec rm \-f {} \; 2>/dev/null

  for i in Config Log WebRoot Monitor ; do
    [[ -e $EPAGES_SHARED/$i ]] || continue
    if mount | grep -q " on $EPAGES_SHARED/$i " ; then
      umount -f $EPAGES_SHARED/$i 2>&1 || { fuser -vm $EPAGES_SHARED/$i; exit 1; }
    fi
    rmdir $EPAGES_SHARED/$i || {
      echo "$BN: Directory must be empty: $EPAGES_SHARED/$i/"
      exit 1
    }
    grep -v "^[^#]* $EPAGES_SHARED/$i " /etc/fstab > /tmp/.~$$~ && cat /tmp/.~$$~ > /etc/fstab
    rm -f /tmp/.~$$~
  done

  if [[ -e $EPAGES_SHARED ]] ; then
    rmdir $EPAGES_SHARED || {
      echo "$BN: Directory must be empty: $EPAGES_SHARED/"
      exit 1
    }
  fi
  if [[ -e $EPAGES_SHARED.nfs_t ]] ; then
    rmdir $EPAGES_SHARED.nfs_t || {
      echo "$BN: Directory must be empty: $EPAGES_SHARED.nfs_t/"
      exit 1
    }
  fi
  mkdir -p -m 755 $EPAGES_SHARED || exit 1
  grep -q "^[^#]* $EPAGES_SHARED " /etc/fstab || {
    echo "$FILESERVER:$EPAGES_SHARED $EPAGES_SHARED nfs $MOUNT_OPT 0 0" >> /etc/fstab
  }
  echo mount $EPAGES_SHARED
  mount $EPAGES_SHARED || exit 1

  ########################################################################
  # setsebool httpd_use_nfs on
  ########################################################################

  if [[ -n $httpd_use_nfs ]] ; then
    echo setsebool httpd_use_nfs on
    setsebool httpd_use_nfs on
  fi

  [[ -z $PERL_CGI ]] && exit 0

  ########################################################################
  # restore EPAGES_CARTRIDGES
  ########################################################################

  if mount | grep -q " on $EPAGES_CARTRIDGES " ; then
    umount -f $EPAGES_CARTRIDGES 2>&1 || { fuser -vm $EPAGES_CARTRIDGES; exit 1; }
  fi
  grep -v "^[^#]* $EPAGES_CARTRIDGES " /etc/fstab > /tmp/.~$$~ && cat /tmp/.~$$~ > /etc/fstab
  rm -f /tmp/.~$$~

  echo "$FILESERVER:$EPAGES_CARTRIDGES $EPAGES_CARTRIDGES nfs $MOUNT_OPT 0 0" >> /etc/fstab

  echo mount $EPAGES_CARTRIDGES
  mount $EPAGES_CARTRIDGES || exit 1

  ########################################################################
  # restore EPAGES_PERL
  ########################################################################

  if mount | grep -q " on $EPAGES_PERL " ; then
    umount -f $EPAGES_PERL 2>&1 || { fuser -vm $EPAGES_PERL; exit 1; }
  fi
  grep -v "^[^#]* $EPAGES_PERL " /etc/fstab > /tmp/.~$$~ && cat /tmp/.~$$~ > /etc/fstab
  rm -f /tmp/.~$$~

  for i in bin lib lib32 man ; do
    [[ -e $EPAGES_PERL/$i ]] || continue
    if mount | grep -q " on $EPAGES_PERL/$i " ; then
      umount -f $EPAGES_PERL/$i 2>&1 || { fuser -vm $EPAGES_PERL/$i; exit 1; }
    fi
    rmdir $EPAGES_PERL/$i || {
      echo "$BN: Directory must be empty: $EPAGES_PERL/$i/"
      exit 1
    }
    grep -v "^[^#]* $EPAGES_PERL/$i " /etc/fstab > /tmp/.~$$~ && cat /tmp/.~$$~ > /etc/fstab
    rm -f /tmp/.~$$~
  done

  if [[ -e $EPAGES_PERL ]] ; then
    rmdir $EPAGES_PERL || {
      echo "$BN: Directory must be empty: $EPAGES_PERL/"
      exit 1
    }
  fi
  mkdir -p -m 755 $EPAGES_PERL || exit 1
  grep -q "^[^#]* $EPAGES_PERL " /etc/fstab || {
    echo "$FILESERVER:$EPAGES_PERL $EPAGES_PERL nfs $MOUNT_OPT 0 0" >> /etc/fstab
  }
  echo mount $EPAGES_PERL
  mount $EPAGES_PERL || exit 1

fi

########################################################################
# start services
########################################################################

if [[ -z $NO_START ]] ; then
  echo /etc/init.d/$EPAGES_PROJECT start
  /etc/init.d/$EPAGES_PROJECT start >/dev/null

  echo /etc/init.d/$EPAGES_PROJECT start_java
  /etc/init.d/$EPAGES_PROJECT start_java >/dev/null
fi

exit 0
